==> dart_icalendar-1.0.1.tar.gz: lib/models/x_date_model.dart <==
import 'dart:convert';

import 'package:dart_icalendar/models/time_zone_model.dart';
import 'package:intl/intl.dart';

class XDate {
  final DateTime date;
  final TimeZone timeZone;

  XDate({
    required this.date,
    required this.timeZone,
  });

  String toICalString() {
    final buffer = StringBuffer();
    buffer.write('EXDATE');
    if (timeZone.isUtc) {
      buffer.write('Z');
      buffer.write(':');
      buffer.write(DateFormat('yyyyMMdd\'T\'HHmmss').format(date));
    } else {
      buffer.write(';TZID=${timeZone.tzId}');
      buffer.write(':');
      buffer.write(DateFormat('yyyyMMdd\'T\'HHmmss').format(date));
    }
    return buffer.toString();
  }

  factory XDate.fromICalString(String icalString) {
    final lines = icalString.split('\n');
    String? dateStr;
    TimeZone? timeZone;

    for (var i = 0; i < lines.length; i++) {
      final line = lines[i];
      if (line.startsWith('EXDATE')) {
        final dateAndTimeZoneStr = line.split(';').last.trim();
        var timeZoneStr = dateAndTimeZoneStr.split(':').length > 1
            ? dateAndTimeZoneStr.split(':').first
            : null;
        timeZoneStr = timeZoneStr?.replaceAll('TZID=', '');
        dateStr = dateAndTimeZoneStr.replaceAll(timeZoneStr ?? '', '').trim();
        dateStr = dateStr.replaceAll('TZID=', '');
        dateStr = dateStr.replaceAll(':', '');
        timeZone = TimeZone(tzId: timeZoneStr ?? 'UTC', transitions: []);

        break;
      }
    }

    if (dateStr == null) {
      throw FormatException('Invalid iCal string. DTSTART not found.');
    }

    final date = DateTime.parse(dateStr);
    return XDate(date: date, timeZone: timeZone ?? TimeZone.utc);
  }

  XDate copyWith({
    DateTime? date,
    TimeZone? timeZone,
  }) {
    return XDate(
      date: date ?? this.date,
      timeZone: timeZone ?? this.timeZone,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'date': date.millisecondsSinceEpoch,
      'timeZone': timeZone.toMap(),
    };
  }

  factory XDate.fromMap(Map<String, dynamic> map) {
    return XDate(
      date: DateTime.fromMillisecondsSinceEpoch(map['date']),
      timeZone: TimeZone.fromMap(map['timeZone']),
    );
  }

  String toJson() => json.encode(toMap());

  factory XDate.fromJson(String source) => XDate.fromMap(json.decode(source));

  @override
  String toString() => 'XDate(date: $date, timeZone: $timeZone)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is XDate && other.date == date && other.timeZone == timeZone;
  }

  @override
  int get hashCode => date.hashCode ^ timeZone.hashCode;
}

==> dart_icalendar-1.0.2.tar.gz: lib/models/x_date_model.dart <==
import 'dart:convert';

import 'package:dart_icalendar/models/time_zone_model.dart';
import 'package:intl/intl.dart';

class XDate {
  final DateTime date;
  final TimeZone timeZone;

  XDate({
    required this.date,
    required this.timeZone,
  });

  String toICalString() {
    final buffer = StringBuffer();
    buffer.write('EXDATE');
    if (timeZone.isUtc) {
      buffer.write('Z');
      buffer.write(':');
      buffer.write(DateFormat('yyyyMMdd\'T\'HHmmss').format(date));
    } else {
      buffer.write(';TZID=${timeZone.tzId}');
      buffer.write(':');
      buffer.write(DateFormat('yyyyMMdd\'T\'HHmmss').format(date));
    }
    return buffer.toString();
  }

  factory XDate.fromICalString(String icalString) {
    final lines = icalString.split('\n');
    String? dateStr;
    TimeZone? timeZone;

    for (var i = 0; i < lines.length; i++) {
      final line = lines[i];
      if (line.startsWith('EXDATE')) {
        final dateAndTimeZoneStr = line.split(';').last.trim();
        var timeZoneStr = dateAndTimeZoneStr.split(':').length > 1
            ? dateAndTimeZoneStr.split(':').first
            : null;
        timeZoneStr = timeZoneStr?.replaceAll('TZID=', '');
        dateStr = dateAndTimeZoneStr.replaceAll(timeZoneStr ?? '', '').trim();
        dateStr = dateStr.replaceAll('TZID=', '');
        dateStr = dateStr.replaceAll(':', '');
        timeZone = TimeZone(tzId: timeZoneStr ?? 'UTC', transitions: []);

        break;
      }
    }

    if (dateStr == null) {
      throw FormatException('Invalid iCal string. DTSTART not found.');
    }

    final date = DateTime.parse(dateStr);
    return XDate(date: date, timeZone: timeZone ?? TimeZone.utc);
  }

  XDate copyWith({
    DateTime? date,
    TimeZone? timeZone,
  }) {
    return XDate(
      date: date ?? this.date,
      timeZone: timeZone ?? this.timeZone,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'date': date.millisecondsSinceEpoch,
      'time_zone': timeZone.toMap(),
    };
  }

  factory XDate.fromMap(Map<String, dynamic> map) {
    return XDate(
      date: DateTime.fromMillisecondsSinceEpoch(map['date']),
      timeZone: TimeZone.fromMap(map['time_zone']),
    );
  }

  String toJson() => json.encode(toMap());

  factory XDate.fromJson(String source) => XDate.fromMap(json.decode(source));

  @override
  String toString() => 'XDate(date: $date, timeZone: $timeZone)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is XDate && other.date == date && other.timeZone == timeZone;
  }

  @override
  int get hashCode => date.hashCode ^ timeZone.hashCode;
}
